import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, Checkbox } from 'antd'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import asyncComponent from '@/utils/asyncComponent'
import MKEmitter from '@/utils/events.js'
import TimerTask from '@/utils/timer-task.js'
import './index.scss'

const CardCellComponent = asyncComponent(() => import('../cardcellList'))

class BalconyComponent extends Component {
  static propTpyes = {
    config: PropTypes.object
  }

  state = {
    BID: '',
    config: null,
    syncConfig: null,
    loading: false,
    data: {},
    BData: null,
    syncData: [],
    show: true,
    checked: false
  }

  loaded = false

  UNSAFE_componentWillMount () {
    const { config } = this.props

    let _config = fromJS(config).toJS()
    let BID = ''
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }
    
    let _data = { $$empty: true }
    if (_config.wrap.datatype === 'dynamic') {
      _config.setting.onload = _config.setting.sync === 'true' ? 'false' : 'true'

      if (_config.setting.sync === 'true' && window.GLOB.SyncData.has(_config.dataName)) {
        _data = window.GLOB.SyncData.get(_config.dataName) || []

        if (_config.$cache) {
          Api.writeCacheConfig(_config.uuid, fromJS(_data).toJS(), BID)
        }

        _config.setting.sync = 'false'

        _data = _data[0] || {$$empty: true}

        this.loaded = true

        window.GLOB.SyncData.delete(_config.dataName)
      }
    } else if (_config.wrap.datatype === 'public') {
      if (window.GLOB.CacheData.has(_config.wrap.publicId)) {
        _data = window.GLOB.CacheData.get(_config.wrap.publicId)
        _data = fromJS(_data).toJS()
      }
    } else {
      _config.elements.forEach(item => {
        if (item.eleType === 'button') return
        if (item.datatype === 'dynamic' && item.field) {
          item.field = item.field.toLowerCase()
        }
      })
    }

    _data.$$BID = BID || ''
    _data.$$BData = BData || ''
    if (_config.setting.primaryKey) {
      _data.$$uuid = _data[_config.setting.primaryKey] || ''
    }

    if (_config.wrap.datatype === 'static' && BData) {
      Object.keys(BData).forEach(key => {
        if (/\$/.test(key)) return
        _data[key.toLowerCase()] = BData[key]
      })
    }

    if (!_config.style.position) {
      if (_config.wrap.position === 'fixed' || _config.wrap.position === 'absolute') {
        _config.style.position = _config.wrap.position
        _config.style.zIndex = _config.wrap.position === 'fixed' ? 3 : 2
        _config.style.left = _config.wrap.left || ''
        _config.style.right = _config.wrap.right || ''
        _config.style.top = _config.wrap.top || ''
        _config.style.bottom = _config.wrap.bottom || ''
        _config.style.transform = _config.wrap.transform || ''
        _config.style.width = _config.wrap.realwidth || ''
      } else {
        _config.style.zIndex = 1
        _config.style.left = _config.wrap.left || ''
        _config.style.right = _config.wrap.right || ''
        _config.style.top = _config.wrap.top || ''
        _config.style.bottom = _config.wrap.bottom || ''
      }
    }

    let show = true
    let syncConfig = null
    if (_config.wrap.linkType === 'sync') {
      syncConfig = _config.syncConfig

      _config.elements = _config.elements.map(item => {
        if (item.eleType === 'button' || item.eleType === 'formula') {
          item.$sync = true
        }
        return item
      })
    } else if (_config.wrap.linkType === 'sup' && _config.wrap.supControl === 'hidden') {
      show = false
    }

    this.setState({
      show,
      syncConfig,
      data: _data,
      BID: BID || '',
      config: _config
    })
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('refreshLineData', this.refreshLineData)
    MKEmitter.addListener('syncBalconyData', this.syncBalconyData)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)

    if (config.setting.sync === 'true') {
      MKEmitter.addListener('transferSyncData', this.transferSyncData)
    }

    if (config.wrap.datatype === 'public') {
      MKEmitter.addListener('mkPublicData', this.mkPublicData)
    }
    
    if (config.wrap.datatype === 'dynamic') {
      if (config.timer) {
        this.timer = new TimerTask()
        this.timer.init(config.uuid, config.timer, config.timerRepeats, () => {this.loadData('timer')})
      }

      this.initExec()
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('mkPublicData', this.mkPublicData)
    MKEmitter.removeListener('refreshLineData', this.refreshLineData)
    MKEmitter.removeListener('syncBalconyData', this.syncBalconyData)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)

    this.timer && this.timer.stop()
  }

  initExec = () => {
    const { config, BID } = this.state

    if (config.$cache) {
      if (config.$time) {
        if (!this.loaded) {
          Api.getLCacheConfig(config.uuid, config.$time, BID).then(res => {
            if (!res.valid && config.setting.onload === 'true') {
              setTimeout(() => {
                this.loadData('init')
              }, config.setting.delay || 0)
            }
  
            if (!res.data || this.loaded) return

            let _data = res.data[0] || {$$empty: true}
            _data.$$uuid = _data[config.setting.primaryKey] || ''
    
            this.setState({data: _data})
          })
        }
      } else {
        if (!this.loaded) {
          Api.getLCacheConfig(config.uuid, 0, BID).then(res => {
            if (!res.data || this.loaded) return
    
            let _data = res.data[0] || {$$empty: true}
            _data.$$uuid = _data[config.setting.primaryKey] || ''
    
            this.setState({data: _data})
          })
        }

        if (config.setting.onload === 'true') {
          setTimeout(() => {
            this.loadData('init')
          }, config.setting.delay || 0)
        }
      }
    } else if (config.setting.onload === 'true') {
      setTimeout(() => {
        this.loadData()
      }, config.setting.delay || 0)
    }
  }

  transferSyncData = (syncId) => {
    const { config } = this.state

    if (config.$syncId !== syncId) return

    const { BID, BData } = this.state

    let _data = window.GLOB.SyncData.get(config.dataName) || []

    if (config.$cache) {
      Api.writeCacheConfig(config.uuid, fromJS(_data).toJS(), BID)
    }

    _data = _data[0] || {$$empty: true}

    _data.$$BID = BID || ''
    _data.$$BData = BData || ''
    _data.$$uuid = _data[config.setting.primaryKey] || ''

    this.loaded = true

    this.setState({data: _data})

    window.GLOB.SyncData.delete(config.dataName)

    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
  }

  mkPublicData = (publicId, data) => {
    const { config, BID, BData } = this.state

    if (config.wrap.datatype === 'public' && config.wrap.publicId === publicId) {
      let _data = fromJS(data).toJS()

      _data.$$BID = BID || ''
      _data.$$BData = BData || ''

      this.setState({data: _data})
    }
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn, id, lines) => {
    const { config, BID, syncConfig } = this.state

    if (config.uuid !== menuId) return

    if (syncConfig) {
      MKEmitter.emit('refreshByButtonResult', syncConfig.uuid, position, btn, id, lines)

      this.loadData()
    } else {
      if (['mainline', 'maingrid', 'popclose'].includes(position) && config.wrap.supModule) {
        MKEmitter.emit('reloadData', config.wrap.supModule, position === 'maingrid' ? '' : BID)
      } else {
        this.loadData()
      }
    }
  }

  refreshLineData = (menuId, btn, uuid, count) => {
    const { config, data } = this.state

    if (config.uuid !== menuId) return
    
    let _data = fromJS(data).toJS()
    _data[btn.field] = count

    this.setState({
      data: _data
    })
  }

  syncBalconyData = (menuId, data, checked) => {
    const { syncConfig } = this.state

    if (!syncConfig || syncConfig.uuid !== menuId) return

    this.setState({syncData: data, checked})
  }

  resetParentParam = (MenuID, id, data) => {
    const { config, syncConfig } = this.state

    if (syncConfig) {
      if (!syncConfig.setting.supModule || syncConfig.setting.supModule !== MenuID) return
  
      if (id !== this.state.BID || id !== '') {
        this.setState({ BID: id, BData: data }, () => {
          this.loadData()
        })
      }
    } else {
      if (!config.wrap.supModule || config.wrap.supModule !== MenuID) return
  
      if (config.wrap.supControl === 'hidden') {
        this.setState({ show: id ? true : false })
      }
  
      if (id !== this.state.BID || id !== '') {
        this.setState({ BID: id, BData: data }, () => {
          this.loadData()
        })
      }
    }
  }

  /**
   * @description 导出Excel时，获取页面搜索排序等参数
   */
  queryModuleParam = (menuId, callback) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    callback({
      orderBy: '',
      search: [],
    })
  }

  reloadData = (menuId) => {
    const { config } = this.state

    if (menuId !== config.uuid) return

    this.loadData()
  }

  async loadData (type) {
    const { config, BID, BData } = this.state

    if (config.wrap.datatype === 'public') {
      MKEmitter.emit('reloadData', config.wrap.publicId)
      return
    }
    
    if (config.wrap.datatype === 'static') {
      let _data = {$$BID: BID || '', $$BData: BData, $$empty: true, $$time: new Date().getTime()}
      if (BData) {
        Object.keys(BData).forEach(key => {
          if (/\$/.test(key)) return
          _data[key.toLowerCase()] = BData[key]
        })
      }

      this.setState({
        data: _data
      })
      return
    } else if (config.setting.supModule && !BID) { // BID 不存在时，不做查询
      this.setState({
        data: {$$BID: BID || '', $$BData: BData, $$empty: true},
      })
      this.loaded = true
      return
    }

    let searches = []

    if (type !== 'timer') {
      this.setState({
        loading: true
      })
    }

    let _orderBy = config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, searches, _orderBy, 1, 1, BID)

    let result = await Api.genericInterface(param)
    if (result.status) {
      let _data = {}

      this.loaded = true
      if (config.$cache && type === 'init') {
        Api.writeCacheConfig(config.uuid, result.data || [], BID)
      }

      if (!result.data || !result.data[0]) {
        _data.$$empty = true
      } else {
        _data = result.data[0]
      }

      _data.$$BID = BID || ''
      _data.$$BData = BData
      _data.$$uuid = _data[config.setting.primaryKey] || ''

      this.setState({
        data: _data,
        loading: false
      })

      if (config.timer && config.clearField && result.data && result.data[0]) {
        let vals = (config.clearValue || '').split(',')
        if (vals.includes(result.data[0][config.clearField])) {
          this.timer && this.timer.stop()
        }
      }

      UtilsDM.querySuccess(result)
    } else {
      this.setState({
        loading: false
      })
      this.timer && this.timer.stop()
      
      UtilsDM.queryFail(result)
    }
  }

  checkAll = (e) => {
    const { syncConfig } = this.state

    MKEmitter.emit('mkCheckAll', syncConfig.uuid, e.target.checked)
  }

  triggerButton = () => {
    const { config, data } = this.state

    if (config.wrap.linkbtn) {
      MKEmitter.emit('triggerBtnId', config.wrap.linkbtn, data.$$empty ? [] : [data])
    }
  }

  render() {
    const { config, loading, data, show, syncConfig, syncData, checked } = this.state

    if (config.wrap.empty === 'hidden' && (!data || data.$$empty)) return null

    let style = {...config.style}
    if (config.wrap.bgField) {
      style.backgroundImage = `url('${data[config.wrap.bgField] || ''}')`
    }
    
    return (
      <div className={'custom-balcony-box' + (!show ? ' hidden' : '')} id={'anchor' + config.uuid} style={style} onClick={this.triggerButton}>
        {loading ?
          <div className="loading-mask" onClick={(e) => e.stopPropagation()}>
            <div className="ant-spin-blur"></div>
            <Spin />
          </div> : null
        }
        {config.wrap.checkAll === 'show' ? <div className="check-all" onClick={(e) => e.stopPropagation()}><Checkbox checked={checked} onChange={this.checkAll}>全选</Checkbox></div> : null}
        <CardCellComponent data={data} syncData={syncData || []} cards={syncConfig || config} cardCell={config} elements={config.elements}/>
      </div>
    )
  }
}

export default BalconyComponent